#!/bin/env python3

'''
TODO / FAIL:

fuentes:
    numeros hex en general:
        **$3D00**
        ''$3D00''
        El valor por defecto de ''CHARS'' es, pues, $3D00 - $0100 = $3C00.

'''

import re
import sys

DEBUG = False
HEX_IN_UPPERCASE = False
LAST_UPDATED = "2023-01-17 18:58:00"

# Expresiones regulares
hex_num = '[\$\&][0-9a-fA-F]{1,4}[hH]?'
dec_num = '[0-9]+'
bin_num = '\%[0-1]+'
flags='NC|C|NZ|Z|M|P|PE|PO';
num = '{}|{}|{}'.format(hex_num,dec_num,bin_num);
mem = '\({}\)'.format(num)
all_regs = "IXh|IXl|IXH|IXL|IYh|IYl|IYH|IYL|AF|BC|DE|HL|IX|IY|SP|A|B|C|D|E|H|L|I|R|F|r|h|l|rr|\((HL|BC|DE|SP)\)";
ix_iy_d = '\(I[XY]\s*[+-]\s*[\dnNdD]+\)'
regs = "(?:{}|{})".format(ix_iy_d,all_regs)
regs_num = "(?:{}|{}|{}|{})".format(ix_iy_d,all_regs,num,mem)
ASM_regexps = [
                '(BIT|SET|RES)\s+\d\s*,\s*{}'.format(regs),
                '(HALT|NOP|CCF|SCF|RLCA|RRCA|RRA|RLA|DJNZ|DAA|RRD|RLD|NEG|CPL|EXX|RETI|RETN|EX\s+[afAF]+,\s*[afAF\']+)[^\w\']',
                'IM\s*[012]',
                '(?:JR|JP|RET|CALL)\s+({})\s*,'.format(flags),
                '(?:JR|JP|CALL)\s*(\((HL|IX|IY)\))?',
                '(LD|ADD|ADC|SUB|SBC)\s+{},\s*{}'.format(regs, regs_num),
                '(LD|ADD|ADC|SUB|SBC)\s+{},\s*["\']'.format(regs),
                '(LD|ADD|ADC|SBC)\s+\({}\),\s*{}'.format(regs, regs_num),
                'LD\s+\({}\),\s*{}'.format(regs_num, regs_num),
                'LD\s+{},'.format(regs),
                '\),\s*{},'.format(regs),
                '(RST|PUSH|POP|INC|DEC|SUB|SBC|RRC|RLC|RRC|SLA|SRA|RRA|RLA|SLL|SLS|SRL|RR|RL|AND|XOR|OR|CP)\s+{}'.format(regs_num),
                'RET\s+(?:{})'.format(flags),
                'CP [\'\"]',
                'IN\s+{},\s*\({}\)'.format(regs_num,regs_num),
                'OUT\s+\s*\({}\),\s*{}'.format(regs_num,regs_num),
                '(RET|LDIR|CPIR|INIR|OTIR|OUTI|OUTD|LDDR|CPDR|INDR|OTDR|LDD|CPD|IND|OTD|LDI|CPI|INI|OTI|DI|EI)[^\w\']',
                'EX\s+(DE|\(SP\)),\s*(HL|IX)',
                '{}'.format(hex_num),
              ]

ld_memdir = '(LD)\s+(\([\w\d]+)\),\s+(\w+)'
ASM_compiled_regsnum = re.compile(ld_memdir)

                # '\'\'(BIT|RES|SET|HALT|NOP|CCF|SCF|RLCA|RRCA|RRA|RLA|DJNZ|DAA|RRD|RLD|NEG|CPL|EXX|RETI|RETN|EX|' +
                # 'IM|JR|JP|RET|CALL|LD|ADD|ADC|SUB|SBC|RST|PUSH|POP|INC|DEC|RRC|RLC|RLC|RRC|SLA|SRA|RRA|RLA|SLL|SLS|' +
                # 'SRL|RR|RL|AND|XOR|OR|CP|RET|IN|OUT|LDIR|CPIR|INIR|OTIR|LDDR|CPDR|INDR|OTDR|LDD|CPD|IND|OTD|LDI|' +
                # 'CPI|INI|OTI|DI|EI)\'\'',

fixes = [
            ["fONT", "FONT"],
            ["aTTR", "ATTR"],
            ["CHaRS", "CHARS"],
            ["FONT_call_JUMP_TABLE", "FONT_CALL_JUMP_TABLE"],
            ["VALor",  "Valor"],
            ["LaST_K", "LAST_K"],
            ["aNCHO", "ANCHO"],
            ["aLTO", "ALTO"],
            ["DM_MaP", "DM_MAP"],
            ["bYTES", "BYTES"],
            ["iniCI", "INICI"],
            ["COLor", "COLOR"],
            ["eQU", "EQU"],
            ["cLOCK", "CLOCK"],
            ["iNTS_PER_SECOND", "iNTS_PER_SECOND"],
            ['dS_COO', 'DS_COO'],
            ['byte2ASCII', 'Byte2ASCII'],
        ]


to_uppercase = [
        '\'\'jr (NC|C|NZ|Z)\'\''
        ];

ASM_compiled_regexps = [re.compile('({})'.format(regexp)) for regexp in ASM_regexps]
to_uppercase_compiled_regexps = [re.compile('({})'.format(regexp)) for regexp in to_uppercase]
hex_regexp = re.compile(hex_num)

# FIX: SetRAMBank => 0x

def Process_Line(line):
    printed = False

    for compiled_regexp in ASM_compiled_regexps:

        for matching_items in re.findall(compiled_regexp, line):

            if ' ^ ' in line or '===' in line:
                print(line, end='')
                return

            if DEBUG and not printed:
                print("\n" + line, end='')
                printed = True

            items = [matching_items] if isinstance(matching_items, str) else matching_items

            match = items[0]
            replacement = match.lower()

            if DEBUG:
                print("\n    => '{}' => '{}'". format(match, replacement))

            line = line.replace(match, replacement)

    # Caso "LD (algo), REG"
    if 'LD (' in line:
        matching_items = re.findall(ASM_compiled_regsnum, line)
        items = [matching_items] if isinstance(matching_items, str) else matching_items
        for item in matching_items:
            line = line.replace(item[0], item[0].lower())
            line = line.replace(item[2], item[2].lower())

    # Listado de fixes manuales
    for fix in fixes:
        if fix[0] in line:
            line = line.replace(fix[0], fix[1])


    # Force uppercase
    for compiled_regexp in to_uppercase_compiled_regexps:

        for matching_items in re.findall(compiled_regexp, line):

            if ' ^ ' in line or '===' in line:
                print(line, end='')
                return

            items = [matching_items] if isinstance(matching_items, str) else matching_items
            match = items[0]
            replacement = match.upper()
            line = line.replace(match, replacement)

    # Devolver los HEX numbers a mayúsculas
    if HEX_IN_UPPERCASE:
        for matching_items in re.findall(hex_regexp, line):
            items = [matching_items] if isinstance(matching_items, str) else matching_items
            line = line.replace(items[0], items[0].upper())

    # Imprimir la línea modificada o original
    print(line, end='')


def Process_File(filename):
    try:
        with open(filename, 'r') as file:
            for line in file:
                Process_Line(line)
    except FileNotFoundError:
        print(f"El archivo '{filename}' no existe.")

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("Uso: python programa.py <nombre_del_archivo>")
    else:
        # Obtener nombre de archivo desde la línea de comandos
        filename = sys.argv[1]
        
        # Verificar que el archivo existe y procesarlo
        Process_File(filename)

